/*
This work is licensed under the 
Creative Commons Attribution-NonCommercial 3.0 Unported License. 
To view a copy of this license, visit 
http://creativecommons.org/licenses/by-nc/3.0/ 
or send a letter to 
Creative Commons, 
444 Castro Street, Suite 900, 
Mountain View, California, 94041, USA.

JBullet library is licensed under the ZLIB licence.
To view a copy of this license, visit 
http://zlib.net/zlib_license.html
http://jbullet.advel.cz/

PeasyCam library is licenced under the 
Apache Public License, version 2.0
To view a copy of this license, visit
http://www.apache.org/licenses/LICENSE-2.0.html
http://mrfeinberg.com/peasycam/

Geosans Light font by Manfred Klein
http://manfred-klein.ina-mar.com/

Richard Brauer -- http://www.r6753.com
*/

import javax.swing.SwingUtilities;

public class DrawObject implements Displayable {
  DrawObject self = DrawObject.this;
  Stoppable runner;
  final long start;
  boolean done;
  
  DrawObject() {
    start = millis();
    init(0);
  }
  DrawObject(final int time) {
    start = millis();
    init(time);
  }
  
  void init(final int time) {
    prep();
    
    runner = new Stoppable() {
      public void run() {
        registerDraw(self);
        while(time>0?millis()-start<time&&go:go) {
          try {
            sleep(time-10<0?10000:time-10);
          } catch(InterruptedException e) {go = false;}
        }
        done = true;
      }
    };
    runner.start();
//    SwingUtilities.invokeLater(
//      new Runnable() {
//        public void run() { runner.start(); }
//      }
//    );
  }
  void prep(){}
  void disp(){}
  void done(){}
  void cease() {runner.interrupt();}
  public void draw() {
    disp();
    if(done) {
      unregisterDraw(self);
      done();
    }
  }
  
  long elapsed() {
    return millis()-start;
  }
  
  abstract class Stoppable extends Thread {
    boolean go = true;
    abstract void run();
    void cease() { go = false; }
  }
}

public interface Displayable {
  public void disp();
}
