/*
This work is licensed under the 
Creative Commons Attribution-NonCommercial 3.0 Unported License. 
To view a copy of this license, visit 
http://creativecommons.org/licenses/by-nc/3.0/ 
or send a letter to 
Creative Commons, 
444 Castro Street, Suite 900, 
Mountain View, California, 94041, USA.

JBullet library is licensed under the ZLIB licence.
To view a copy of this license, visit 
http://zlib.net/zlib_license.html
http://jbullet.advel.cz/

PeasyCam library is licenced under the 
Apache Public License, version 2.0
To view a copy of this license, visit
http://www.apache.org/licenses/LICENSE-2.0.html
http://mrfeinberg.com/peasycam/

Geosans Light font by Manfred Klein
http://manfred-klein.ina-mar.com/

Richard Brauer -- http://www.r6753.com
*/

class Cam extends DrawObject {
  PeasyCam pc;
  PVector look, pos, delta;
  float rdir, lightb;
  boolean light = true;
  
  Cam(PApplet p5, double distance) {
    pc = new PeasyCam(p5, distance);
    pc.setResetOnDoubleClick(false);
    pc.setMouseControlled(false);
    rdir = lightb = 0;
    look = new PVector();
    pos = new PVector();
    delta = new PVector();
  }
  
  public void disp() {
    camread();
    if(light) {
      directionalLight(
        lightb,lightb,lightb, delta.x,delta.y,delta.z
      );
      pointLight(lightb*.1,lightb*.1,lightb*.2, 0,-500,0);
    }
//    fill(164,196,255);
    fill(132,164,196);
    pc.beginHUD();
    rect(0,0,width,height);
    pc.endHUD();
  }
  
  void camread() {
    look.set(pc.getLookAt());
    pos.set(pc.getPosition());
    delta = PVector.sub(look, pos);
    delta.normalize();
  }
  void camrot(float x, float y, float ry, Vector3D look, 
    float speed
  ) {
    rdir = alerp(rdir, atan2(y, x), speed);
    pc.setState(
      new CameraState(
        new Rotation(
          RotationOrder.YXZ,
          rdir, ry, 0),
        new Vector3D(look.getX(), look.getY(), look.getZ()),
        pc.getDistance()
      ), 
      0
    );
  }
  
  void camlight(float in, float speed) {
    lightb = alerp(lightb, in, speed);
  }
  
  float alerp(float in, float tar, float rate) {
    if(abs(in-tar)<.001) return tar;
    float dt = tar-in;
    return in+dt*rate;
  }
  
}
