/*
This work is licensed under the 
Creative Commons Attribution-NonCommercial 3.0 Unported License. 
To view a copy of this license, visit 
http://creativecommons.org/licenses/by-nc/3.0/ 
or send a letter to 
Creative Commons, 
444 Castro Street, Suite 900, 
Mountain View, California, 94041, USA.

JBullet library is licensed under the ZLIB licence.
To view a copy of this license, visit 
http://zlib.net/zlib_license.html
http://jbullet.advel.cz/

PeasyCam library is licenced under the 
Apache Public License, version 2.0
To view a copy of this license, visit
http://www.apache.org/licenses/LICENSE-2.0.html
http://mrfeinberg.com/peasycam/

Geosans Light font by Manfred Klein
http://manfred-klein.ina-mar.com/

Richard Brauer -- http://www.r6753.com
*/

class Body extends RigidBody {
  DynamicsWorld world;
  DrawObject drawer;
  Transform wtrans;
  RigidBodyConstructionInfo ci;
  
  Body(
    DynamicsWorld world, CollisionShape shp, 
    float mass, Vector3f trans
  ) {
    this(world, shp, mass, trans, true);
  }
  Body(
    DynamicsWorld world, CollisionShape shp, 
    float mass, Vector3f trans, boolean inertia
  ) {
    super(UsingCI.from(shp, mass, trans, inertia));
    this.world = world;
    wtrans = new Transform();
  }
  
  void updateTrans() {
    wtrans = getMotionState().getWorldTransform(wtrans);
  }
  Matrix3f getBasis() {
    updateTrans();
    return wtrans.basis;
  }
  Vector3f getPos() {
    updateTrans();
    return wtrans.origin;
  }

  void addUsing(DrawObject drawer) {
    this.drawer = drawer;
    world.addRigidBody(this);
  }

}

static class UsingCI {   
  static RigidBodyConstructionInfo from(
    CollisionShape shp, float mass, Vector3f trans, 
    boolean inertia
  ) {
    Transform ctrans = new Transform();
    if(trans!=null) ctrans.origin.set(trans);
    
    Vector3f ivect = new Vector3f(0,0,0);
    if(inertia)
      shp.calculateLocalInertia(mass, ivect);
      
    DefaultMotionState dms =
      trans!=null?
        new DefaultMotionState(ctrans):
        new DefaultMotionState();
        
    return new RigidBodyConstructionInfo(mass, dms, shp, ivect);
  }
}
