/*
This work is licensed under the 
Creative Commons Attribution-NonCommercial 3.0 Unported License. 
To view a copy of this license, visit 
http://creativecommons.org/licenses/by-nc/3.0/ 
or send a letter to 
Creative Commons, 
444 Castro Street, Suite 900, 
Mountain View, California, 94041, USA.

JBullet library is licensed under the ZLIB licence.
To view a copy of this license, visit 
http://zlib.net/zlib_license.html
http://jbullet.advel.cz/

PeasyCam library is licenced under the 
Apache Public License, version 2.0
To view a copy of this license, visit
http://www.apache.org/licenses/LICENSE-2.0.html
http://mrfeinberg.com/peasycam/

Geosans Light font by Manfred Klein
http://manfred-klein.ina-mar.com/

Richard Brauer -- http://www.r6753.com
*/

class QuadGrid {
  ArrayList<PVector> data;
  ArrayList<int[]> quads;
  float cellw, cellh;
  int ww, hh, vv;
  
  QuadGrid(float cellw, float cellh, int ww, int hh) {
    this.cellw = cellw;
    this.cellh = cellh;
    this.ww = ww;
    this.hh = hh;
    vv = (ww+1)*(hh+1);
    data = new ArrayList();
    quads = new ArrayList();
    
    init();
  }
  
  void init() {
    float startx = -cellw*ww*.5;
    float startz = -cellh*hh*.5;
    int row = 0;
    int col = 0;
    
    for(int a=0; a<vv; a++) {
      if(a<(ww+1)*hh && row!=ww-1) {
        quads.add(
          new int[]{a, a+1, a+ww+2, a+ww+1}
        );
      }
      row = a%(ww+1);
      col = floor(a/(ww+1));
      data.add(new PVector(startx+cellw*row, 0, startz+cellh*col));
    }
  }
  
  boolean centerIn(float ww, float hh, int[] qind) {
    float cx = 0;
    float cz = 0;
    for(int ind : qind) {
      PVector cur = data.get(ind);
      cx += cur.x;
      cz += cur.z;
    }
    cx *= .25;
    cz *= .25;
    return cx>-ww*.5 && cx<ww*.5 && cz>-hh*.5 && cz<hh*.5;
  }
  
  int[] triInds() {
    int[] ret = new int[0];
    for(int[] cur : quads) {
      ret = (int[])append(ret, cur[0]);
      ret = (int[])append(ret, cur[1]);
      ret = (int[])append(ret, cur[2]);
      ret = (int[])append(ret, cur[2]);
      ret = (int[])append(ret, cur[3]);
      ret = (int[])append(ret, cur[0]);
    }
    return ret;
  }

}
